@@ -65,7 +65,7 @@ main (void)
   /* connect to the peer */
   sd = udp_connect ();
 
-  gnutls_transport_set_ptr (session, (gnutls_transport_ptr_t) sd);
+  gnutls_transport_set_int (session, sd);
 
   /* set the connection MTU */
   gnutls_dtls_set_mtu (session, 1000);
@@ -76,7 +76,8 @@ main (void)
     {
       ret = gnutls_handshake (session);
     }
-  while (ret < 0 && gnutls_error_is_fatal (ret) == 0);
+  while (ret == GNUTLS_E_INTERRUPTED || ret == GNUTLS_E_AGAIN);
+  /* Note that DTLS may also receive GNUTLS_E_LARGE_PACKET */
 
   if (ret < 0)
     {
@@ -86,7 +87,11 @@ main (void)
     }
   else
     {
-      printf ("- Handshake was completed\n");
+      char* desc;
+      
+      desc = gnutls_session_get_desc(session);
+      printf ("- Session info: %s\n", desc);
+      gnutls_free(desc);
     }
 
   gnutls_record_send (session, MSG, strlen (MSG));
