@@ -49,6 +49,7 @@ struct cryptodev_ctx
   struct session_op sess;
   struct crypt_op cryp;
   uint8_t iv[EALG_MAX_BLOCK_LEN];
+  int reset;
 
   int cfd;
 };
@@ -351,9 +352,11 @@ int ret;
 static const gnutls_crypto_mac_st mac_struct = {
   .init = NULL,
   .setkey = NULL,
+  .setnonce = NULL,
   .hash = NULL,
   .output = NULL,
   .deinit = NULL,
+  .reset = NULL,
   .fast = cryptodev_mac_fast
 };
 
@@ -404,6 +407,7 @@ static const gnutls_crypto_digest_st digest_struct = {
   .hash = NULL,
   .output = NULL,
   .deinit = NULL,
+  .reset = NULL,
   .fast = cryptodev_digest_fast
 };
 
