@@ -7,7 +7,7 @@
  *
  * The GnuTLS is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
- * as published by the Free Software Foundation; either version 3 of
+ * as published by the Free Software Foundation; either version 2.1 of
  * the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful, but
@@ -71,6 +71,7 @@ int gnutls_random_art (gnutls_random_art_t type,
 
 /* ANON & DHE */
 
+#if defined(ENABLE_DHE) || defined(ENABLE_ANON)
 /**
  * gnutls_dh_set_prime_bits:
  * @session: is a #gnutls_session_t structure.
@@ -88,14 +89,20 @@ int gnutls_random_art (gnutls_random_art_t type,
  * Note that values lower than 512 bits may allow decryption of the
  * exchanged data.
  *
- * This function has no effect in server side.
+ * The function has no effect in server side.
+ * 
+ * Note that since 3.1.7 this function is deprecated. The minimum
+ * number of bits is set by the priority string level.
+ * Also this function must be called after gnutls_priority_set_direct()
+ * or the set value may be overriden by the selected priority options.
+ *
  *
  **/
 void
 gnutls_dh_set_prime_bits (gnutls_session_t session, unsigned int bits)
 {
   if (bits <= 512) _gnutls_audit_log(session, "Note that the security level of the Diffie-Hellman key exchange has been lowered to %u bits and this may allow decryption of the session data\n", bits);
-  session->internals.dh_prime_bits = bits;
+  session->internals.priorities.dh_prime_bits = bits;
 }
 
 
@@ -224,57 +231,6 @@ gnutls_dh_get_pubkey (gnutls_session_t session, gnutls_datum_t * raw_key)
                             dh->public_key.size);
 }
 
-/**
- * gnutls_rsa_export_get_pubkey:
- * @session: is a gnutls session
- * @exponent: will hold the exponent.
- * @modulus: will hold the modulus.
- *
- * This function will return the peer's public key exponent and
- * modulus used in the last RSA-EXPORT authentication.  The output
- * parameters must be freed with gnutls_free().
- *
- * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise
- *   an error code is returned.
- **/
-int
-gnutls_rsa_export_get_pubkey (gnutls_session_t session,
-                              gnutls_datum_t * exponent,
-                              gnutls_datum_t * modulus)
-{
-  cert_auth_info_t info;
-  int ret;
-
-  if (gnutls_auth_get_type (session) == GNUTLS_CRD_CERTIFICATE)
-    {
-      info = _gnutls_get_auth_info (session);
-      if (info == NULL)
-        return GNUTLS_E_INTERNAL_ERROR;
-
-      ret = _gnutls_set_datum (modulus, info->rsa_export.modulus.data,
-                               info->rsa_export.modulus.size);
-      if (ret < 0)
-        {
-          gnutls_assert ();
-          return ret;
-        }
-
-      ret = _gnutls_set_datum (exponent, info->rsa_export.exponent.data,
-                               info->rsa_export.exponent.size);
-      if (ret < 0)
-        {
-          gnutls_assert ();
-          _gnutls_free_datum (modulus);
-          return ret;
-        }
-
-      return 0;
-    }
-
-  return GNUTLS_E_INVALID_REQUEST;
-}
-
-
 /**
  * gnutls_dh_get_secret_bits:
  * @session: is a gnutls session
@@ -325,6 +281,7 @@ gnutls_dh_get_secret_bits (gnutls_session_t session)
     }
 }
 
+
 static int
 mpi_buf2bits (gnutls_datum_t * mpi_buf)
 {
@@ -404,26 +361,6 @@ gnutls_dh_get_prime_bits (gnutls_session_t session)
   return mpi_buf2bits (&dh->prime);
 }
 
-/**
- * gnutls_rsa_export_get_modulus_bits:
- * @session: is a gnutls session
- *
- * Get the export RSA parameter's modulus size.
- *
- * Returns: The bits used in the last RSA-EXPORT key exchange with the
- *   peer, or a negative error code in case of error.
- **/
-int
-gnutls_rsa_export_get_modulus_bits (gnutls_session_t session)
-{
-  cert_auth_info_t info;
-
-  info = _gnutls_get_auth_info (session);
-  if (info == NULL)
-    return GNUTLS_E_INTERNAL_ERROR;
-
-  return mpi_buf2bits (&info->rsa_export.modulus);
-}
 
 /**
  * gnutls_dh_get_peers_public_bits:
@@ -483,6 +420,27 @@ gnutls_dh_get_peers_public_bits (gnutls_session_t session)
   return mpi_buf2bits (&dh->public_key);
 }
 
+/**
+ * gnutls_certificate_set_dh_params:
+ * @res: is a gnutls_certificate_credentials_t structure
+ * @dh_params: is a structure that holds Diffie-Hellman parameters.
+ *
+ * This function will set the Diffie-Hellman parameters for a
+ * certificate server to use. These parameters will be used in
+ * Ephemeral Diffie-Hellman cipher suites.  Note that only a pointer
+ * to the parameters are stored in the certificate handle, so you
+ * must not deallocate the parameters before the certificate is deallocated.
+ *
+ **/
+void
+gnutls_certificate_set_dh_params (gnutls_certificate_credentials_t res,
+                                  gnutls_dh_params_t dh_params)
+{
+  res->dh_params = dh_params;
+}
+
+#endif /* DH */
+
 /* CERTIFICATE STUFF */
 
 /**
@@ -552,6 +510,7 @@ gnutls_certificate_get_peers (gnutls_session_t
   return info->raw_certificate_list;
 }
 
+#ifdef ENABLE_OPENPGP
 /**
  * gnutls_certificate_get_peers_subkey_id:
  * @session: is a gnutls session
@@ -562,6 +521,8 @@ gnutls_certificate_get_peers (gnutls_session_t
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise
  *   an error code is returned.
+ *
+ * Since: 3.1.3
  **/
 int gnutls_certificate_get_peers_subkey_id(gnutls_session_t session,
                                            gnutls_datum_t *id)
@@ -579,6 +540,7 @@ int gnutls_certificate_get_peers_subkey_id(gnutls_session_t session,
 
   return 0;
 }
+#endif
 
 /**
  * gnutls_certificate_client_get_request_status:
@@ -610,7 +572,7 @@ gnutls_certificate_client_get_request_status (gnutls_session_t session)
  *
  * This is the usual way to calculate a fingerprint of an X.509 DER
  * encoded certificate.  Note however that the fingerprint of an
- * OpenPGP is not just a hash and cannot be calculated with this
+ * OpenPGP certificate is not just a hash and cannot be calculated with this
  * function.
  *
  * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise
@@ -641,27 +603,6 @@ gnutls_fingerprint (gnutls_digest_algorithm_t algo,
   return 0;
 }
 
-
-/**
- * gnutls_certificate_set_dh_params:
- * @res: is a gnutls_certificate_credentials_t structure
- * @dh_params: is a structure that holds Diffie-Hellman parameters.
- *
- * This function will set the Diffie-Hellman parameters for a
- * certificate server to use. These parameters will be used in
- * Ephemeral Diffie-Hellman cipher suites.  Note that only a pointer
- * to the parameters are stored in the certificate handle, so if you
- * deallocate the parameters before the certificate is deallocated,
- * you must change the parameters stored in the certificate first.
- *
- **/
-void
-gnutls_certificate_set_dh_params (gnutls_certificate_credentials_t res,
-                                  gnutls_dh_params_t dh_params)
-{
-  res->dh_params = dh_params;
-}
-
 /**
  * gnutls_certificate_set_params_function:
  * @res: is a gnutls_certificate_credentials_t structure
@@ -678,7 +619,6 @@ gnutls_certificate_set_params_function (gnutls_certificate_credentials_t res,
   res->params_func = func;
 }
 
-
 /**
  * gnutls_certificate_set_verify_flags:
  * @res: is a gnutls_certificate_credentials_t structure
@@ -716,22 +656,7 @@ gnutls_certificate_set_verify_limits (gnutls_certificate_credentials_t res,
   res->verify_bits = max_bits;
 }
 
-/**
- * gnutls_certificate_set_rsa_export_params:
- * @res: is a gnutls_certificate_credentials_t structure
- * @rsa_params: is a structure that holds temporary RSA parameters.
- *
- * This function will set the temporary RSA parameters for a
- * certificate server to use.  These parameters will be used in
- * RSA-EXPORT cipher suites.
- **/
-void
-gnutls_certificate_set_rsa_export_params (gnutls_certificate_credentials_t
-                                          res, gnutls_rsa_params_t rsa_params)
-{
-  res->rsa_params = rsa_params;
-}
-
+#ifdef ENABLE_PSK
 /**
  * gnutls_psk_set_params_function:
  * @res: is a gnutls_psk_server_credentials_t structure
@@ -747,7 +672,9 @@ gnutls_psk_set_params_function (gnutls_psk_server_credentials_t res,
 {
   res->params_func = func;
 }
+#endif
 
+#ifdef ENABLE_ANON
 /**
  * gnutls_anon_set_params_function:
  * @res: is a gnutls_anon_server_credentials_t structure
@@ -763,6 +690,7 @@ gnutls_anon_set_params_function (gnutls_anon_server_credentials_t res,
 {
   res->params_func = func;
 }
+#endif
 
 /**
  * gnutls_load_file:
@@ -800,32 +728,6 @@ size_t len;
   return 0;
 }
 
-/**
- * gnutls_url_is_supported:
- * @url: A PKCS 11 url
- *
- * Check whether url is supported.  Depending on the system libraries
- * GnuTLS may support pkcs11 or tpmkey URLs.
- *
- * Returns: return non-zero if the given URL is supported, and zero if
- * it is not known.
- *
- * Since: 3.1.0
- **/
-int
-gnutls_url_is_supported (const char* url)
-{
-#ifdef ENABLE_PKCS11
-  if (strstr(url, "pkcs11:") != NULL)
-    return 1;
-#endif
-#ifdef HAVE_TROUSERS
-  if (strstr(url, "tpmkey:") != NULL)
-    return 1;
-#endif
-  return 0;
-}
-
 /**
  * gnutls_ocsp_status_request_is_checked:
  * @session: is a gnutls session
@@ -845,3 +747,125 @@ gnutls_ocsp_status_request_is_checked (gnutls_session_t session, unsigned int fl
 {
   return session->internals.ocsp_check_ok;
 }
+
+#ifdef ENABLE_RSA_EXPORT
+
+/**
+ * gnutls_rsa_export_get_pubkey:
+ * @session: is a gnutls session
+ * @exponent: will hold the exponent.
+ * @modulus: will hold the modulus.
+ *
+ * This function will return the peer's public key exponent and
+ * modulus used in the last RSA-EXPORT authentication.  The output
+ * parameters must be freed with gnutls_free().
+ *
+ * Returns: On success, %GNUTLS_E_SUCCESS (0) is returned, otherwise
+ *   an error code is returned.
+ **/
+int
+gnutls_rsa_export_get_pubkey (gnutls_session_t session,
+                              gnutls_datum_t * exponent,
+                              gnutls_datum_t * modulus)
+{
+  return gnutls_assert_val(GNUTLS_E_UNIMPLEMENTED_FEATURE);
+}
+
+/**
+ * gnutls_rsa_export_get_modulus_bits:
+ * @session: is a gnutls session
+ *
+ * Get the export RSA parameter's modulus size.
+ *
+ * Returns: The bits used in the last RSA-EXPORT key exchange with the
+ *   peer, or a negative error code in case of error.
+ **/
+int
+gnutls_rsa_export_get_modulus_bits (gnutls_session_t session)
+{
+  return gnutls_assert_val(GNUTLS_E_UNIMPLEMENTED_FEATURE);
+}
+
+/**
+ * gnutls_certificate_set_rsa_export_params:
+ * @res: is a gnutls_certificate_credentials_t structure
+ * @rsa_params: is a structure that holds temporary RSA parameters.
+ *
+ * This function will set the temporary RSA parameters for a
+ * certificate server to use.  These parameters will be used in
+ * RSA-EXPORT cipher suites.
+ **/
+void
+gnutls_certificate_set_rsa_export_params (gnutls_certificate_credentials_t
+                                          res, gnutls_rsa_params_t rsa_params)
+{
+  return;
+}
+#endif
+
+#define DESC_SIZE 64
+
+/**
+ * gnutls_session_get_desc:
+ * @session: is a gnutls session
+ *
+ * This function returns a string describing the current session.
+ * The string is null terminated and allocated using gnutls_malloc().
+ *
+ * Returns: a description of the protocols and algorithms in the current session.
+ *
+ * Since: 3.1.10
+ **/
+char *
+gnutls_session_get_desc (gnutls_session_t session)
+{
+    gnutls_kx_algorithm_t kx;
+    unsigned type;
+    char kx_name[32];
+    char proto_name[32];
+    const char* curve_name = NULL;
+    unsigned dh_bits = 0;
+    char* desc;
+
+    kx = session->security_parameters.kx_algorithm;
+    
+    if (kx == GNUTLS_KX_ANON_ECDH || kx == GNUTLS_KX_ECDHE_PSK ||
+        kx == GNUTLS_KX_ECDHE_RSA || kx == GNUTLS_KX_ECDHE_ECDSA) 
+      {
+        curve_name = gnutls_ecc_curve_get_name(gnutls_ecc_curve_get(session));
+      }
+    else if (kx == GNUTLS_KX_ANON_DH || kx == GNUTLS_KX_DHE_PSK ||
+        kx == GNUTLS_KX_DHE_RSA || kx == GNUTLS_KX_DHE_DSS) 
+      {
+        dh_bits = gnutls_dh_get_prime_bits (session);
+      }
+      
+    if (curve_name != NULL)
+      snprintf(kx_name, sizeof(kx_name), "%s-%s", gnutls_kx_get_name(kx), curve_name);
+    else if (dh_bits != 0)
+      snprintf(kx_name, sizeof(kx_name), "%s-%u", gnutls_kx_get_name(kx), dh_bits);
+    else
+      snprintf(kx_name, sizeof(kx_name), "%s", gnutls_kx_get_name(kx));
+
+    type = gnutls_certificate_type_get (session);
+    if (type == GNUTLS_CRT_X509)
+      snprintf(proto_name, sizeof(proto_name), "%s-PKIX", gnutls_protocol_get_name(_gnutls_protocol_get_version(session)));
+    else
+      snprintf(proto_name, sizeof(proto_name), "%s-%s", gnutls_protocol_get_name(_gnutls_protocol_get_version(session)),
+                                                        gnutls_certificate_type_get_name(type));
+    
+    gnutls_protocol_get_name(_gnutls_protocol_get_version (session)),
+
+    desc = gnutls_malloc(DESC_SIZE);
+    if (desc == NULL)
+      return NULL;
+
+    snprintf(desc, DESC_SIZE,
+             "(%s)-(%s)-(%s)-(%s)",
+            proto_name,
+            kx_name,
+            gnutls_cipher_get_name (gnutls_cipher_get (session)),
+            gnutls_mac_get_name (gnutls_mac_get (session)));
+            
+    return desc;
+}
