@@ -30,10 +30,12 @@
 #include <nettle/md2.h>
 #include <nettle/sha.h>
 #include <nettle/hmac.h>
+#include <nettle/umac.h>
 
 typedef void (*update_func) (void *, unsigned, const uint8_t *);
 typedef void (*digest_func) (void *, unsigned, uint8_t *);
 typedef void (*set_key_func) (void *, unsigned, const uint8_t *);
+typedef void (*set_nonce_func) (void *, unsigned, const uint8_t *);
 
 static int wrap_nettle_hash_init (gnutls_digest_algorithm_t algo, void **_ctx);
 
@@ -56,7 +58,7 @@ struct nettle_hash_ctx
   digest_func digest;
 };
 
-struct nettle_hmac_ctx
+struct nettle_mac_ctx
 {
   union
   {
@@ -66,101 +68,125 @@ struct nettle_hmac_ctx
     struct hmac_sha384_ctx sha384;
     struct hmac_sha512_ctx sha512;
     struct hmac_sha1_ctx sha1;
+    struct umac96_ctx umac96;
+    struct umac128_ctx umac128;
   } ctx;
   
-  /* this is the context just after
-   * the set_key. Used in reset().
-   */
-  union
-  {
-    struct hmac_md5_ctx md5;
-    struct hmac_sha224_ctx sha224;
-    struct hmac_sha256_ctx sha256;
-    struct hmac_sha384_ctx sha384;
-    struct hmac_sha512_ctx sha512;
-    struct hmac_sha1_ctx sha1;
-  } init_ctx;
   void *ctx_ptr;
   gnutls_mac_algorithm_t algo;
   size_t length;
   update_func update;
   digest_func digest;
-  set_key_func setkey;
+  set_key_func set_key;
+  set_nonce_func set_nonce;
 };
 
-static int _hmac_ctx_init(gnutls_mac_algorithm_t algo, struct nettle_hmac_ctx *ctx)
+static void
+_wrap_umac96_set_key(void* ctx, unsigned len, const uint8_t* key)
 {
+	return umac96_set_key(ctx, key);
+}
+
+static void
+_wrap_umac128_set_key(void* ctx, unsigned len, const uint8_t* key)
+{
+	return umac128_set_key(ctx, key);
+}
+
+static int _mac_ctx_init(gnutls_mac_algorithm_t algo, struct nettle_mac_ctx *ctx)
+{
+  ctx->set_nonce = NULL;
   switch (algo)
     {
     case GNUTLS_MAC_MD5:
       ctx->update = (update_func) hmac_md5_update;
       ctx->digest = (digest_func) hmac_md5_digest;
-      ctx->setkey = (set_key_func) hmac_md5_set_key;
+      ctx->set_key = (set_key_func) hmac_md5_set_key;
       ctx->ctx_ptr = &ctx->ctx.md5;
       ctx->length = MD5_DIGEST_SIZE;
       break;
     case GNUTLS_MAC_SHA1:
       ctx->update = (update_func) hmac_sha1_update;
       ctx->digest = (digest_func) hmac_sha1_digest;
-      ctx->setkey = (set_key_func) hmac_sha1_set_key;
+      ctx->set_key = (set_key_func) hmac_sha1_set_key;
       ctx->ctx_ptr = &ctx->ctx.sha1;
       ctx->length = SHA1_DIGEST_SIZE;
       break;
     case GNUTLS_MAC_SHA224:
       ctx->update = (update_func) hmac_sha224_update;
       ctx->digest = (digest_func) hmac_sha224_digest;
-      ctx->setkey = (set_key_func) hmac_sha224_set_key;
+      ctx->set_key = (set_key_func) hmac_sha224_set_key;
       ctx->ctx_ptr = &ctx->ctx.sha224;
       ctx->length = SHA224_DIGEST_SIZE;
       break;
     case GNUTLS_MAC_SHA256:
       ctx->update = (update_func) hmac_sha256_update;
       ctx->digest = (digest_func) hmac_sha256_digest;
-      ctx->setkey = (set_key_func) hmac_sha256_set_key;
+      ctx->set_key = (set_key_func) hmac_sha256_set_key;
       ctx->ctx_ptr = &ctx->ctx.sha256;
       ctx->length = SHA256_DIGEST_SIZE;
       break;
     case GNUTLS_MAC_SHA384:
       ctx->update = (update_func) hmac_sha384_update;
       ctx->digest = (digest_func) hmac_sha384_digest;
-      ctx->setkey = (set_key_func) hmac_sha384_set_key;
+      ctx->set_key = (set_key_func) hmac_sha384_set_key;
       ctx->ctx_ptr = &ctx->ctx.sha384;
       ctx->length = SHA384_DIGEST_SIZE;
       break;
     case GNUTLS_MAC_SHA512:
       ctx->update = (update_func) hmac_sha512_update;
       ctx->digest = (digest_func) hmac_sha512_digest;
-      ctx->setkey = (set_key_func) hmac_sha512_set_key;
+      ctx->set_key = (set_key_func) hmac_sha512_set_key;
       ctx->ctx_ptr = &ctx->ctx.sha512;
       ctx->length = SHA512_DIGEST_SIZE;
       break;
+    case GNUTLS_MAC_UMAC_96:
+      ctx->update = (update_func) umac96_update;
+      ctx->digest = (digest_func) umac96_digest;
+      ctx->set_key = _wrap_umac96_set_key;
+      ctx->set_nonce = (set_nonce_func) umac96_set_nonce;
+      ctx->ctx_ptr = &ctx->ctx.umac96;
+      ctx->length = 12;
+      break;
+    case GNUTLS_MAC_UMAC_128:
+      ctx->update = (update_func) umac128_update;
+      ctx->digest = (digest_func) umac128_digest;
+      ctx->set_key = _wrap_umac128_set_key;
+      ctx->set_nonce = (set_nonce_func) umac128_set_nonce;
+      ctx->ctx_ptr = &ctx->ctx.umac128;
+      ctx->length = 16;
+      break;
     default:
       gnutls_assert ();
       return GNUTLS_E_INVALID_REQUEST;
     }
-  
+
   return 0;
 }
 
-static int wrap_nettle_hmac_fast(gnutls_mac_algorithm_t algo, 
-  const void *key, size_t key_size, const void* text, size_t text_size, 
+static int wrap_nettle_mac_fast(gnutls_mac_algorithm_t algo, 
+  const void* nonce, size_t nonce_size,
+  const void *key, size_t key_size, 
+  const void* text, size_t text_size, 
   void* digest)
 {
-  struct nettle_hmac_ctx ctx;
+  struct nettle_mac_ctx ctx;
   int ret;
   
-  ret = _hmac_ctx_init(algo, &ctx);
+  ret = _mac_ctx_init(algo, &ctx);
   if (ret < 0)
     return gnutls_assert_val(ret);
 
-  ctx.setkey (&ctx, key_size, key);
+  if (ctx.set_nonce)
+    ctx.set_nonce (&ctx, nonce_size, nonce);
+  ctx.set_key (&ctx, key_size, key);
   ctx.update (&ctx, text_size, text);
   ctx.digest (&ctx, ctx.length, digest);
   
   return 0;
 }
 
-static int wrap_nettle_hmac_exists(gnutls_mac_algorithm_t algo)
+static int wrap_nettle_mac_exists(gnutls_mac_algorithm_t algo)
 {
   switch (algo)
     {
@@ -170,6 +196,8 @@ static int wrap_nettle_hmac_exists(gnutls_mac_algorithm_t algo)
     case GNUTLS_MAC_SHA256:
     case GNUTLS_MAC_SHA384:
     case GNUTLS_MAC_SHA512:
+    case GNUTLS_MAC_UMAC_96:
+    case GNUTLS_MAC_UMAC_128:
       return 1;
     default:
       return 0;
@@ -177,12 +205,12 @@ static int wrap_nettle_hmac_exists(gnutls_mac_algorithm_t algo)
 }
 
 static int
-wrap_nettle_hmac_init (gnutls_mac_algorithm_t algo, void **_ctx)
+wrap_nettle_mac_init (gnutls_mac_algorithm_t algo, void **_ctx)
 {
-  struct nettle_hmac_ctx *ctx;
+  struct nettle_mac_ctx *ctx;
   int ret;
 
-  ctx = gnutls_calloc (1, sizeof (struct nettle_hmac_ctx));
+  ctx = gnutls_calloc (1, sizeof (struct nettle_mac_ctx));
   if (ctx == NULL)
     {
       gnutls_assert ();
@@ -191,7 +219,7 @@ wrap_nettle_hmac_init (gnutls_mac_algorithm_t algo, void **_ctx)
 
   ctx->algo = algo;
 
-  ret = _hmac_ctx_init(algo, ctx);
+  ret = _mac_ctx_init(algo, ctx);
   if (ret < 0)
     {
       gnutls_free(ctx);
@@ -204,29 +232,32 @@ wrap_nettle_hmac_init (gnutls_mac_algorithm_t algo, void **_ctx)
 }
 
 static int
-wrap_nettle_hmac_setkey (void *_ctx, const void *key, size_t keylen)
+wrap_nettle_mac_set_key (void *_ctx, const void *key, size_t keylen)
 {
-  struct nettle_hmac_ctx *ctx = _ctx;
+  struct nettle_mac_ctx *ctx = _ctx;
 
-  ctx->setkey (ctx->ctx_ptr, keylen, key);
+  ctx->set_key (ctx->ctx_ptr, keylen, key);
   
-  memcpy(&ctx->init_ctx, &ctx->ctx, sizeof(ctx->ctx));
+  return GNUTLS_E_SUCCESS;
+}
+
+static int
+wrap_nettle_mac_set_nonce (void *_ctx, const void *nonce, size_t noncelen)
+{
+  struct nettle_mac_ctx *ctx = _ctx;
+  
+  if (ctx->set_nonce == NULL)
+    return gnutls_assert_val(GNUTLS_E_INVALID_REQUEST);
+
+  ctx->set_nonce (ctx->ctx_ptr, noncelen, nonce);
 
   return GNUTLS_E_SUCCESS;
 }
 
-static void
-wrap_nettle_hmac_reset (void *_ctx)
-{
-  struct nettle_hmac_ctx *ctx = _ctx;
-
-  memcpy(&ctx->ctx, &ctx->init_ctx, sizeof(ctx->ctx));
-}
-
 static int
-wrap_nettle_hmac_update (void *_ctx, const void *text, size_t textsize)
+wrap_nettle_mac_update (void *_ctx, const void *text, size_t textsize)
 {
-  struct nettle_hmac_ctx *ctx = _ctx;
+  struct nettle_mac_ctx *ctx = _ctx;
 
   ctx->update (ctx->ctx_ptr, textsize, text);
 
@@ -234,9 +265,9 @@ wrap_nettle_hmac_update (void *_ctx, const void *text, size_t textsize)
 }
 
 static int
-wrap_nettle_hmac_output (void *src_ctx, void *digest, size_t digestsize)
+wrap_nettle_mac_output (void *src_ctx, void *digest, size_t digestsize)
 {
-  struct nettle_hmac_ctx *ctx;
+  struct nettle_mac_ctx *ctx;
   ctx = src_ctx;
 
   if (digestsize < ctx->length)
@@ -251,7 +282,7 @@ wrap_nettle_hmac_output (void *src_ctx, void *digest, size_t digestsize)
 }
 
 static void
-wrap_nettle_hmac_deinit (void *hd)
+wrap_nettle_mac_deinit (void *hd)
 {
   gnutls_free (hd);
 }
@@ -413,31 +444,20 @@ wrap_nettle_hash_output (void *src_ctx, void *digest, size_t digestsize)
   return 0;
 }
 
-static void
-wrap_nettle_hash_reset (void *src_ctx)
-{
-  struct nettle_hash_ctx *ctx;
-  ctx = src_ctx;
-
-  _ctx_init(ctx->algo, ctx->ctx_ptr);
-}
-
-
 gnutls_crypto_mac_st _gnutls_mac_ops = {
-  .init = wrap_nettle_hmac_init,
-  .setkey = wrap_nettle_hmac_setkey,
-  .hash = wrap_nettle_hmac_update,
-  .reset = wrap_nettle_hmac_reset,
-  .output = wrap_nettle_hmac_output,
-  .deinit = wrap_nettle_hmac_deinit,
-  .fast = wrap_nettle_hmac_fast,
-  .exists = wrap_nettle_hmac_exists,
+  .init = wrap_nettle_mac_init,
+  .setkey = wrap_nettle_mac_set_key,
+  .setnonce = wrap_nettle_mac_set_nonce,
+  .hash = wrap_nettle_mac_update,
+  .output = wrap_nettle_mac_output,
+  .deinit = wrap_nettle_mac_deinit,
+  .fast = wrap_nettle_mac_fast,
+  .exists = wrap_nettle_mac_exists,
 };
 
 gnutls_crypto_digest_st _gnutls_digest_ops = {
   .init = wrap_nettle_hash_init,
   .hash = wrap_nettle_hash_update,
-  .reset = wrap_nettle_hash_reset,
   .output = wrap_nettle_hash_output,
   .deinit = wrap_nettle_hash_deinit,
   .fast = wrap_nettle_hash_fast,
