@@ -32,7 +32,6 @@
 #include <locks.h>
 #include <gnutls_num.h>
 #include <nettle/yarrow.h>
-#include <timespec.h>
 #ifdef HAVE_GETPID
 # include <unistd.h> /* getpid */
 #endif
@@ -93,11 +92,7 @@ do_trivia_source (int init)
 
   memcpy(&event.now, &current_time, sizeof(event.now));
 #ifdef HAVE_GETRUSAGE
-#ifdef RUSAGE_THREAD
-  if (getrusage (RUSAGE_THREAD, &event.rusage) < 0)
-#else
   if (getrusage (RUSAGE_SELF, &event.rusage) < 0)
-#endif
     {
       _gnutls_debug_log ("getrusage failed: %s\n", strerror (errno));
       abort ();
